@@ -60,6 +60,8 @@ gda_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 		g_error (_("libgda needs GModule. Finishing..."));
 
 	initialized = TRUE;
+
+	g_message (_("libgda successfully initialized"));
 }
 
 typedef struct {
@@ -111,7 +113,7 @@ gda_main_run (GdaInitFunc init_func, gpointer user_data)
 	}
 
 	main_loop = g_main_loop_new (g_main_context_default (), FALSE);
-	g_main_run (main_loop);
+	g_main_loop_run (main_loop);
 }
 
 /**
